@@ -25,8 +25,9 @@ void logEvent(const XEvent &xev) {
     switch (xev.type) {
 #if 1
     case CreateNotify:
-        msg("window=0x%lX: create parent=0x%lX, (%d:%d-%dx%d) border_width=%d, override_redirect=%s",
+        msg("window=0x%lX: create serial=%10d parent=0x%lX, (%d:%d-%dx%d) border_width=%d, override_redirect=%s",
             xev.xcreatewindow.window,
+            xev.xany.serial,
             xev.xcreatewindow.parent,
             xev.xcreatewindow.x, xev.xcreatewindow.y,
             xev.xcreatewindow.width, xev.xcreatewindow.height,
@@ -35,8 +36,9 @@ void logEvent(const XEvent &xev) {
         break;
 
     case DestroyNotify:
-        msg("window=0x%lX: destroy event=0x%lX",
+        msg("window=0x%lX: destroy serial=%10d event=0x%lX",
             xev.xdestroywindow.window,
+            xev.xany.serial,
             xev.xdestroywindow.event);
         break;
 #else
@@ -46,8 +48,9 @@ void logEvent(const XEvent &xev) {
 #endif
 #if 1
     case MapRequest:
-        msg("window=0x%lX: mapRequest parent=0x%lX",
+        msg("window=0x%lX: mapRequest serial=%10d parent=0x%lX",
             xev.xmaprequest.window,
+            xev.xany.serial,
             xev.xmaprequest.parent);
         break;
 #else
@@ -56,15 +59,17 @@ void logEvent(const XEvent &xev) {
 #endif
 #if 1
     case MapNotify:
-        msg("window=0x%lX: mapNotify event=0x%lX, override_redirect=%s",
+        msg("window=0x%lX: mapNotify serial=%10d event=0x%lX, override_redirect=%s",
             xev.xmap.window,
+            xev.xany.serial,
             xev.xmap.event,
             xev.xmap.override_redirect ? "True" : "False");
         break;
 
     case UnmapNotify:
-        msg("window=0x%lX: unmapNotify event=0x%lX, from_configure=%s send_event=%s",
+        msg("window=0x%lX: unmapNotify serial=%10d event=0x%lX, from_configure=%s send_event=%s",
             xev.xunmap.window,
+            xev.xany.serial,
             xev.xunmap.event,
             xev.xunmap.from_configure ? "True" : "False",
             xev.xunmap.from_configure ? "True" : "False");
@@ -76,9 +81,10 @@ void logEvent(const XEvent &xev) {
 #endif
 #if 1
     case ConfigureRequest:
-        msg("window=0x%lX: %s configureRequest parent=0x%lX, (%d:%d-%dx%d) border_width=%d, above=0x%lX, detail=%d, value_mask=0x%lX",
+        msg("window=0x%lX: %s configureRequest serial=%10d parent=0x%lX, (%d:%d-%dx%d) border_width=%d, above=0x%lX, detail=%d, value_mask=0x%lX",
             xev.xconfigurerequest.window,
             xev.xconfigurerequest.send_event ? "synth" : "real",
+            xev.xany.serial,
             xev.xconfigurerequest.parent,
             xev.xconfigurerequest.x, xev.xconfigurerequest.y,
             xev.xconfigurerequest.width, xev.xconfigurerequest.height,
@@ -122,8 +128,9 @@ void logEvent(const XEvent &xev) {
 
 #if 1
     case ReparentNotify:
-        msg("window=0x%lX: reparentNotify event=0x%lX, parent=0x%lX, (%d:%d), override_redirect=%s",
+        msg("window=0x%lX: reparentNotify serial=%10d event=0x%lX, parent=0x%lX, (%d:%d), override_redirect=%s",
             xev.xreparent.window,
+            xev.xany.serial,
             xev.xreparent.event,
             xev.xreparent.parent,
             xev.xreparent.x, xev.xreparent.y,
@@ -136,8 +143,9 @@ void logEvent(const XEvent &xev) {
 
 #if 1
     case ConfigureNotify:
-        msg("window=0x%lX: configureNotify event=0x%lX, (%d:%d-%dx%d) border_width=%d, above=0x%lX, override_redirect=%s",
+        msg("window=0x%lX: configureNotify serial=%10d event=0x%lX, (%d:%d-%dx%d) border_width=%d, above=0x%lX, override_redirect=%s",
             xev.xconfigure.window,
+            xev.xany.serial,
             xev.xconfigure.event,
             xev.xconfigure.x, xev.xconfigure.y,
             xev.xconfigure.width, xev.xconfigure.height,
