@@ -859,45 +859,23 @@ void dumpZorder(const char *oper, YFrameWindow *w, YFrameWindow *a) {
 }
 #endif
 
-void runRestart(const char *path, char *const *args) {
+void YWMApp::runRestart(const char *path, char *const *args) {
     XSelectInput(xapp->display(), desktop->handle(), 0);
     XSync(xapp->display(), False);
     ///!!! problem with repeated SIGHUP for restart...
-    app->resetSignals();
+    resetSignals();
 
-#ifdef linux   /* for now, some debugging code */
-        {
-            /* close all files */
-
-            int             i, max = 1024;
-            struct rlimit   lim;
-
-            if (getrlimit(RLIMIT_NOFILE, &lim) == 0)
-                max = lim.rlim_max;
-
-            for (i = 3; i < max; i++) {
-                int fl;
-                if (fcntl(i, F_GETFD, &fl) == 0) {
-                    if (!(fl & FD_CLOEXEC)) {
-                        warn("file descriptor still open: %d. "
-                             " Check /proc/$icewmpid/fd/%d when running next time. "
-                             "Please report a bug (perhaps not an icewm problem)!", i, i);
-                    }
-                }
-                close (i);
-            }
-        }
-#endif
+    closeFiles();
 
     if (path) {
         if (args) {
             execvp(path, args);
         } else {
-            execlp(path, path, NULL);
+            execlp(path, path, (void *)NULL);
         }
     } else {
         const char *c = configArg ? "-c" : NULL;
-        execlp(ICEWMEXE, ICEWMEXE, "--restart", c, configArg, NULL);
+        execlp(ICEWMEXE, ICEWMEXE, "--restart", c, configArg, (void *)NULL);
     }
 
     xapp->alert();
@@ -932,7 +910,7 @@ void YWMApp::runOnce(const char *resource, const char *path, char *const *args)
 }
 
 void YWMApp::runCommandOnce(const char *resource, const char *cmdline) {
-#warning calling /bin/sh is considered to be bloat
+/// TODO #warning calling /bin/sh is considered to be bloat
     char const *const argv[] = { "/bin/sh", "-c", cmdline, NULL };
 
     if (resource)
@@ -1316,7 +1294,7 @@ void YWMApp::handleSignal(int sig) {
 
 void YWMApp::handleIdle() {
 #ifdef CONFIG_TASKBAR
-#warning "make this generic"
+/// TODO #warning "make this generic"
     if (taskBar) {
         taskBar->relayoutNow();
     }
