@@ -11,7 +11,7 @@ public:
     CtrlAltDelete(YWindow *parent);
     virtual ~CtrlAltDelete();
 
-    virtual void paint(Graphics &g, int x, int y, unsigned int width, unsigned int height);
+    virtual void paint(Graphics &g, const YRect &r);
     virtual bool handleKey(const XKeyEvent &key);
     virtual void actionPerformed(YAction *action, unsigned int modifiers);
 
@@ -28,10 +28,10 @@ private:
 
 extern CtrlAltDelete *ctrlAltDelete; // !!! remove
 
-extern YPixmap *logoutPixmap;
+extern ref<YPixmap> logoutPixmap;
 
 #ifdef CONFIG_GRADIENTS
-extern class YPixbuf *logoutPixbuf;
+extern ref<YPixbuf> logoutPixbuf;
 #endif
 
 #endif
