@@ -366,7 +366,7 @@ void YFrameWindow::doManage(YFrameClient *clientw) {
             } else if (net_wm_window_type ==
                        _XA_NET_WM_WINDOW_TYPE_DESKTOP)
             {
-#warning "this needs some cleanup"
+/// TODO #warning "this needs some cleanup"
                 setSticky(true);
                 setTypeDesktop(true);
                 updateMwmHints();
@@ -376,6 +376,7 @@ void YFrameWindow::doManage(YFrameClient *clientw) {
                 setTypeSplash(true);
                 updateMwmHints();
             }
+            updateLayer(true);
         } else if (fClient->getWinLayerHint(&layer))
             setRequestedLayer(layer);
 #endif
@@ -439,7 +440,7 @@ void YFrameWindow::doManage(YFrameClient *clientw) {
     }
 
     if (client()->getNetWMDesktopHint(&workspace)) {
-        if (workspace == 0xFFFFFFFF)
+        if (workspace == (long)0xFFFFFFFF)
             setSticky(true);
         else
             setWorkspace(workspace);
@@ -568,7 +569,7 @@ void YFrameWindow::manage(YFrameClient *client) {
     PRECONDITION(client != 0);
     fClient = client;
 
-#warning "optimize this, do it only if needed"
+/// TODO #warning "optimize this, do it only if needed"
     XSetWindowBorderWidth(xapp->display(),
                           client->handle(),
                           0);
@@ -776,7 +777,7 @@ void YFrameWindow::configureClient(const XConfigureRequestEvent &configureReques
 #if 1
                     { /* warning, tcl/tk "fix" here */
                         XEvent xev;
-#warning "looks like sendConfigure but not quite, investigate!"
+/// TODO #warning "looks like sendConfigure but not quite, investigate!"
 
                         memset(&xev, 0, sizeof(xev));
                         xev.xconfigure.type = ConfigureNotify;
@@ -820,7 +821,7 @@ void YFrameWindow::configureClient(const XConfigureRequestEvent &configureReques
 void YFrameWindow::configureClient(int cx, int cy, int cwidth, int cheight) {
     MSG(("setting geometry (%d:%d %dx%d)", cx, cy, cwidth, cheight));
     cy -= titleYN();
-#warning "alternative configure mechanism would be nice"
+/// TODO #warning "alternative configure mechanism would be nice"
     if (isFullscreen())
         return;
     else {
@@ -1043,7 +1044,7 @@ void YFrameWindow::insertFocusFrame(bool focus) {
         setPrevFocus(manager->lastFocusFrame());
         manager->setLastFocusFrame(this);
     } else {
-#warning "XXX: insert as next focus, not as last focus"
+/// TODO #warning "XXX: insert as next focus, not as last focus"
         if (manager->firstFocusFrame())
             manager->firstFocusFrame()->setPrevFocus(this);
         else
@@ -1633,41 +1634,44 @@ void YFrameWindow::focusOnMap() {
         return ;
     }
 
-    if (owner() != 0) {
-        if (focusOnMapTransient)
-            if (owner()->focused() || !focusOnMapTransientActive)
-           {
-               if (fDelayFocusTimer) {
-                   fDelayFocusTimer->stopTimer();
-                   fDelayFocusTimer->setTimerListener(0);
-               }
-               if (fAutoRaiseTimer) {
-                   fAutoRaiseTimer->stopTimer();
-                   fAutoRaiseTimer->setTimerListener(0);
-               }
-               activate();
-           }
-    } else {
-        if (::focusOnMap)
-        {
-            if (fDelayFocusTimer) {
-                fDelayFocusTimer->stopTimer();
-                fDelayFocusTimer->setTimerListener(0);
+
+    if (!(frameOptions() & foNoFocusOnMap)) {
+        if (owner() != 0) {
+            if (focusOnMapTransient) {
+                if (owner()->focused() || !focusOnMapTransientActive)
+                {
+                    if (fDelayFocusTimer) {
+                        fDelayFocusTimer->stopTimer();
+                        fDelayFocusTimer->setTimerListener(0);
+                    }
+                    if (fAutoRaiseTimer) {
+                        fAutoRaiseTimer->stopTimer();
+                        fAutoRaiseTimer->setTimerListener(0);
+                    }
+                    activate();
+                }
             }
-            if (fAutoRaiseTimer) {
-                fAutoRaiseTimer->stopTimer();
-                fAutoRaiseTimer->setTimerListener(0);
-            }
-            activate();
         } else {
-            setWmUrgency(true);
+            if (::focusOnMap) {
+                if (fDelayFocusTimer) {
+                    fDelayFocusTimer->stopTimer();
+                    fDelayFocusTimer->setTimerListener(0);
+                }
+                if (fAutoRaiseTimer) {
+                    fAutoRaiseTimer->stopTimer();
+                    fAutoRaiseTimer->setTimerListener(0);
+                }
+                activate();
+            } else {
+                setWmUrgency(true);
+            }
         }
     }
 }
 
 void YFrameWindow::wmShow() {
     // recover lost (offscreen) windows !!! (unify with code below)
-#warning "this is really broken"
+/// TODO #warning "this is really broken"
     if (x() >= int(manager->width()) ||
         y() >= int(manager->height()) ||
         x() <= - int(width()) ||
@@ -1692,7 +1696,7 @@ void YFrameWindow::wmShow() {
 }
 
 void YFrameWindow::focus(bool canWarp) {
-#warning "move focusChangesWorkspace check out of here, to (some) callers"
+/// TODO #warning "move focusChangesWorkspace check out of here, to (some) callers"
     manager->lockFocus();
     if (!visibleOn(manager->activeWorkspace())) {
         manager->activateWorkspace(getWorkspace());
@@ -2082,7 +2086,7 @@ void YFrameWindow::getFrameHints() {
     if (win_hints & WinHintsDoNotCover)
         fFrameOptions |= foDoNotCover;
 
-#warning "need initial window mapping cleanup"
+/// TODO #warning "need initial window mapping cleanup"
     if (fTypeDesktop) {
         fFrameDecors = 0;
         fFrameOptions |= foIgnoreTaskBar;
@@ -2309,7 +2313,7 @@ void YFrameWindow::updateIcon() {
     Pixmap *pixmap;
     Atom type;
 
-#warning "think about winoptions specified icon here"
+/// TODO #warning "think about winoptions specified icon here"
 
     YIcon *oldFrameIcon(fFrameIcon);
 
@@ -2756,11 +2760,11 @@ void YFrameWindow::setNormalGeometryInner(int x, int y, int w, int h) {
     normalW = sh ? (w - sh->base_width) / sh->width_inc : w;
     normalH = sh ? (h - sh->base_height) / sh->height_inc : h ;
 
-    updateDerivedSize();
+    updateDerivedSize((isMaximizedVert() ? WinStateMaximizedVert : 0) | (isMaximizedHoriz() ? WinStateMaximizedHoriz : 0));
     updateLayout();
 }
 
-void YFrameWindow::updateDerivedSize() {
+void YFrameWindow::updateDerivedSize(long flagmask) {
     XSizeHints *sh = client()->sizeHints();
 
     int nx = normalX;
@@ -2795,14 +2799,14 @@ void YFrameWindow::updateDerivedSize() {
         }
     }
 
-    if (isMaximizedHoriz()) {
+    if (isMaximizedHoriz() && (flagmask & WinStateMaximizedHoriz)) {
         nw = Mw;
         if (considerHorizBorder) {
             nw -= 2 * borderXN();
         }
     }
 
-    if (isMaximizedVert()) {
+    if (isMaximizedVert() && (flagmask & WinStateMaximizedVert)) {
         nh = Mh;
         if (considerVertBorder) {
             nh -= 2 * borderYN();
@@ -2841,13 +2845,28 @@ void YFrameWindow::updateDerivedSize() {
         nh += 2 * borderYN();
     }
 
-    posX = nx;
-    posY = ny;
-    posW = nw;
-    posH = nh + titleYN();
+    bool cx = true;
+    bool cy = true;
+    bool cw = true;
+    bool ch = true;
+
+    if (isMaximizedVert() && !(flagmask & WinStateMaximizedVert))
+        cy = ch = false;
+    if (isMaximizedHoriz() && !(flagmask & WinStateMaximizedHoriz))
+        cx = cw = false;
+
+    if (cx)
+        posX = nx;
+    if (cy)
+        posY = ny;
+    if (cw)
+        posW = nw;
+    if (ch)
+        posH = nh + titleYN();
 }
 
 void YFrameWindow::updateNormalSize() {
+    MSG(("updateNormalSize: %d %d %d %d", normalX, normalY, normalW, normalH));
     XSizeHints *sh = client()->sizeHints();
 
     bool cx = true;
@@ -2876,9 +2895,12 @@ void YFrameWindow::updateNormalSize() {
         normalH = posH - (2 * borderYN() + titleYN());
         normalH = sh ? (normalH - sh->base_height) / sh->height_inc : normalH;
     }
+    MSG(("updateNormalSize> %d %d %d %d", normalX, normalY, normalW, normalH));
 }
 
 void YFrameWindow::setCurrentGeometryOuter(YRect newSize) {
+    MSG(("setCurrentGeometryOuter: %d %d %d %d",
+         newSize.x(), newSize.y(), newSize.width(), newSize.height()));
     setWindowGeometry(newSize);
 
     bool cx = true;
@@ -2906,6 +2928,8 @@ void YFrameWindow::setCurrentGeometryOuter(YRect newSize) {
     }
 
     updateNormalSize();
+    MSG(("setCurrentGeometryOuter> %d %d %d %d",
+         posX, posY, posW, posH));
 }
 
 void YFrameWindow::setCurrentPositionOuter(int x, int y) {
@@ -2934,6 +2958,8 @@ void YFrameWindow::updateLayout() {
             if (isRollup()) {
                 setWindowGeometry(YRect(posX, posY, posW, 2 * borderY() + titleY()));
             } else {
+                MSG(("updateLayout %d %d %d %d",
+                     posX, posY, posW, posH));
                 setWindowGeometry(YRect(posX, posY, posW, posH));
             }
         }
@@ -2975,7 +3001,6 @@ void YFrameWindow::setState(long mask, long state) {
                 fMaximizeButton->setToolTip(_("Maximize"));
             }
         }
-        updateDerivedSize();
     }
     if ((fOldState ^ fNewState) & WinStateMinimized) {
         MSG(("WinStateMinimized: %d", isMaximized()));
@@ -3027,6 +3052,7 @@ void YFrameWindow::setState(long mask, long state) {
     updateState();
     updateLayer();
     manager->updateFullscreenLayer();
+    updateDerivedSize(fOldState ^ fNewState);
     updateLayout();
 
 #ifdef CONFIG_SHAPE
