@@ -56,6 +56,9 @@ YWindowManager::YWindowManager(YWindow *parent, Window win):
     fWorkArea = 0;
     fWorkAreaCount = 0;
     fFullscreenEnabled = true;
+    fFocusedWindow = new YFrameWindow *[workspaceCount()];
+    for (int w = 0; w < workspaceCount(); w++)
+        fFocusedWindow[w] = 0;
 
     frameContext = XUniqueContext();
     clientContext = XUniqueContext();
@@ -177,6 +180,10 @@ void YWindowManager::grabKeys() {
     GRAB_WMKEY(gKeySysHideAll);
 
     GRAB_WMKEY(gKeySysShowDesktop);
+#ifdef CONFIG_TASKBAR
+    if (taskBar != 0)
+        GRAB_WMKEY(gKeySysCollapseTaskBar);
+#endif
 
 #ifndef NO_CONFIGURE_MENUS
     {
@@ -456,7 +463,14 @@ bool YWindowManager::handleWMKey(const XKeyEvent &key, KeySym k, unsigned int /*
     } else if(IS_WMKEY(k, vm, gKeySysShowDesktop)) {
         XAllowEvents(xapp->display(), AsyncKeyboard, key.time);
         wmapp->actionPerformed(actionShowDesktop, 0);
-        return true;                            
+        return true;
+    } else if(IS_WMKEY(k, vm, gKeySysCollapseTaskBar)) {
+        XAllowEvents(xapp->display(), AsyncKeyboard, key.time);
+#ifdef CONFIG_TASKBAR
+        if (taskBar)
+            taskBar->handleCollapseButton();
+#endif
+        return true;
     } else {
 #ifndef NO_CONFIGURE_MENUS
         KProgram *p = keyProgs;
@@ -497,6 +511,7 @@ bool YWindowManager::handleKey(const XKeyEvent &key) {
         KeySym k = XKeycodeToKeysym(xapp->display(), key.keycode, 0);
         unsigned int m = KEY_MODMASK(key.state);
 
+        (void)m;
 #ifdef DEBUG
         MSG(("up key: %d, mod: %d", k, m));
 #endif
@@ -840,7 +855,9 @@ void YWindowManager::setFocus(YFrameWindow *f, bool /*canWarp*/) {
     updateFullscreenLayer();
 }
 
+/// TODO lose this function
 void YWindowManager::loseFocus(YFrameWindow *window) {
+    (void)window;
     PRECONDITION(window != 0);
     focusLastWindow();
 }
@@ -1387,7 +1404,7 @@ YFrameWindow *YWindowManager::manageClient(Window win, bool mapClient) {
             posWidth = min(posWidth, Mw);
             posHeight = min(posHeight, Mh);
 
-#warning "cleanup the constrainSize code, there is some duplication"
+/// TODO #warning "cleanup the constrainSize code, there is some duplication"
             posHeight -= frame->titleYN();
             frame->client()->constrainSize(posWidth, posHeight, 0);
             posHeight += frame->titleYN();
@@ -1551,28 +1568,34 @@ YFrameWindow *YWindowManager::getLastFocus(long workspace) {
 
     YFrameWindow *toFocus = 0;
 
-    for (int pass = 0; pass < 2; pass++) {
-        for (YFrameWindow *w = lastFocusFrame();
-             w;
-             w = w->prevFocus())
-        {
-            if ((w->client() && !w->client()->adopted()))
-                continue;
-            if (w->isMinimized())
-                continue;
-            if (w->isHidden())
-                continue;
-            if (!w->isFocusable(true)) {
-            } else if (0 && w->isSticky()) {
-#warning "this creates more problems than it solves"
-                if (pass == 1) {
-                    toFocus = w;
-                    goto gotit;
-                }
-            } else if (w->getWorkspace() != workspace /* && !w->isSticky()*/) {
-                continue;
-            } else {
-                if (pass == 0) {
+    toFocus = fFocusedWindow[workspace];
+    if (toFocus != 0) {
+        if (toFocus->isMinimized() ||
+            toFocus->isHidden() ||
+            !toFocus->isFocusable(true) ||
+            !toFocus->visibleOn(workspace))
+            toFocus = 0;
+    }
+
+    if (toFocus == 0) {
+        for (int pass = 0; pass < 2; pass++) {
+            for (YFrameWindow *w = lastFocusFrame();
+                 w;
+                 w = w->prevFocus())
+            {
+#if 1
+                if ((w->client() && !w->client()->adopted()))
+                    continue;
+#endif
+                if (w->isMinimized())
+                    continue;
+                if (w->isHidden())
+                    continue;
+                if (!w->isFocusable(true))
+                    continue;
+                if (!w->visibleOn(workspace))
+                    continue;
+                if (!w->isSticky() || pass == 1) {
                     toFocus = w;
                     goto gotit;
                 }
@@ -1609,7 +1632,7 @@ void YWindowManager::focusLastWindow() {
         return ;
     }
 
-#warning "per workspace?"
+/// TODO #warning "per workspace?"
     YFrameWindow *toFocus = getLastFocus();
 
     if (toFocus == 0) {
@@ -1796,7 +1819,7 @@ void YWindowManager::getWorkArea(const YFrameWindow *frame,
         *My = height();
     } else {
 
-#warning "rewrite workarea determine code (per workspace)"
+/// TODO #warning "rewrite workarea determine code (per workspace)"
 #if 1
         *mx = fWorkArea[ws].fMinX;
         *my = fWorkArea[ws].fMinY;
@@ -1977,7 +2000,7 @@ void YWindowManager::announceWorkArea() {
 }
 
 void YWindowManager::relocateWindows(long workspace, int dx, int dy) {
-#warning "needs a rewrite (save old work area) for each workspace"
+/// TODO #warning "needs a rewrite (save old work area) for each workspace"
 #if 1
     for (YFrameWindow * f = topLayer(); f; f = f->nextLayer())
         if (f->inWorkArea()) {
@@ -1994,7 +2017,7 @@ void YWindowManager::resizeWindows() {
     for (YFrameWindow * f = topLayer(); f; f = f->nextLayer()) {
         if (f->inWorkArea()) {
             if (f->isMaximized())
-                f->updateDerivedSize();
+                f->updateDerivedSize(0);
                 f->updateLayout();
         }
     }
@@ -2077,7 +2100,7 @@ void YWindowManager::activateWorkspace(long workspace) {
 #ifndef LITE
         if (workspaceSwitchStatus
 #ifdef CONFIG_TASKBAR
-            && (!showTaskBar || !taskBarShowWorkspaces)
+            && (!showTaskBar || !taskBarShowWorkspaces || taskBarAutoHide)
 #endif
            )
             statusWorkspace->begin(workspace);
@@ -2119,7 +2142,7 @@ void YWindowManager::getIconPosition(YFrameWindow *frame, int *iconX, int *iconY
     y += iw->height();
     if (y >= My) {
         x += iw->width();
-        y = Mx;
+        y = my;
         if (x >= Mx) {
             x = 0;
             y = 0;
@@ -2277,6 +2300,11 @@ void YWindowManager::removeClientFrame(YFrameWindow *frame) {
             if (fArrangeInfo[i].frame == frame)
                 fArrangeInfo[i].frame = 0;
     }
+    for (int w = 0; w < workspaceCount(); w++) {
+        if (fFocusedWindow[w] == frame) {
+            fFocusedWindow[w] = frame->nextLayer();
+        }
+    }
     if (frame == getFocus())
         manager->loseFocus(frame);
     if (frame == getFocus())
@@ -2306,6 +2334,8 @@ void YWindowManager::switchFocusTo(YFrameWindow *frame, bool reorderFocus) {
         ///msg("setting %lX", fFocusWin);
         if (fFocusWin)
             fFocusWin->setWinFocus();
+
+        fFocusedWindow[activeWorkspace()] = frame;
     }
     if (frame && frame->nextFocus() && reorderFocus) {
         frame->removeFocusFrame();
@@ -2375,9 +2405,6 @@ void YWindowManager::switchToWorkspace(long nw, bool takeCurrent) {
         } else {
             activateWorkspace(nw);
         }
-#ifdef CONFIG_TASKBAR
-        if (taskBar) taskBar->popOut();
-#endif
     }
 }
 
@@ -2670,7 +2697,7 @@ void YWindowManager::handleRRScreenChangeNotify(const XRRScreenChangeNotifyEvent
             taskBar->updateLocation();
         }
 #endif
-#warning "make something better"
+/// TODO #warning "make something better"
         wmapp->actionPerformed(actionArrange, 0);
     }
 }
