@@ -39,6 +39,7 @@
 #include "yrect.h"
 #include "yxtray.h"
 #include "prefs.h"
+#include "yicon.h"
 
 #include "aapm.h"
 
@@ -56,6 +57,9 @@ static YColor *taskBarBg = 0;
 static ref<YIconImage> icewmImage;
 static ref<YIconImage> windowsImage;
 static ref<YIconImage> showDesktopImage;
+static ref<YIconImage> collapseImage;
+static ref<YIconImage> expandImage;
+
 /// TODO #warning "these should be static/elsewhere"
 ref<YPixmap> taskbackPixmap;
 #ifdef CONFIG_GRADIENTS
@@ -91,6 +95,8 @@ static void initPixmaps() {
 
     windowsImage = subdirs.loadImage(base, "windows.xpm");
     showDesktopImage = subdirs.loadImage(base, "desktop.xpm");
+    collapseImage = subdirs.loadImage(base, "collapse.xpm");
+    expandImage = subdirs.loadImage(base, "expand.xpm");
 
 #ifdef CONFIG_GRADIENTS
     if (taskbackPixbuf == null)
@@ -230,7 +236,7 @@ YFrameClient(aParent, 0) INIT_GRADIENT(fGradient, NULL)
     setPointer(YXApplication::leftPointer);
     setDND(true);
 
-    fAutoHideTimer = new YTimer(autoHideDelay);
+    fAutoHideTimer = new YTimer(autoShowDelay);
     if (fAutoHideTimer) {
         fAutoHideTimer->setTimerListener(this);
     }
@@ -396,6 +402,7 @@ void TaskBar::initApplets() {
         fCollapseButton = new YButton(this, actionCollapseTaskbar);
         if (fCollapseButton) {
             fCollapseButton->setText(">");
+            fCollapseButton->setImage(collapseImage);
             fCollapseButton->setActionListener(this);
         }
     } else
@@ -535,10 +542,10 @@ void TaskBar::updateLayout(int &size_w, int &size_h) {
 #endif
 #ifdef CONFIG_APPLET_NET_STATUS
 #ifdef CONFIG_APPLET_MAILBOX
-        { fNetStatus ? fNetStatus[0] : 0, false, 1, true, 1, 1, false },
+        { fNetStatus ? fNetStatus[0] : 0, false, 1, false, 1, 1, false },
 /// TODO #warning "a hack"
-        { fNetStatus && fNetStatus[0] ? fNetStatus[1] : 0, false, 1, true, 1, 1, false },
-        { fNetStatus && fNetStatus[0] && fNetStatus[1] ? fNetStatus[2] : 0, false, 1, true, 1, 1, false },
+        { fNetStatus && fNetStatus[0] ? fNetStatus[1] : 0, false, 1, false, 1, 1, false },
+        { fNetStatus && fNetStatus[0] && fNetStatus[1] ? fNetStatus[2] : 0, false, 1, false, 1, 1, false },
 #endif
 #endif
 #ifdef CONFIG_APPLET_APM
@@ -592,6 +599,9 @@ void TaskBar::updateLayout(int &size_w, int &size_h) {
 
     right[0] = w;
     right[1] = w;
+    if (taskBarShowWindows && fTasks != 0) {
+        h[0] = YIcon::smallSize() + 8;
+    }
 
     for (i = 0; wl = wlist + i, i < wcount; i++) {
         if (wl->w == 0)
@@ -770,12 +780,12 @@ void TaskBar::handleCrossing(const XCrossingEvent &crossing) {
     if (crossing.type == EnterNotify /* && crossing.mode != NotifyNormal */) {
         fIsHidden = false;
         if (taskBarAutoHide && fAutoHideTimer)
-            fAutoHideTimer->startTimer();
+            fAutoHideTimer->startTimer(autoShowDelay);
     } else if (crossing.type == LeaveNotify /* && crossing.mode != NotifyNormal */) {
         if (crossing.detail != NotifyInferior) {
             fIsHidden = taskBarAutoHide;
             if (taskBarAutoHide && fAutoHideTimer)
-                fAutoHideTimer->startTimer();
+                fAutoHideTimer->startTimer(autoHideDelay);
         }
     }
 }
@@ -793,7 +803,7 @@ void TaskBar::handleEndPopup(YPopupWindow *popup) {
     if (!hasPopup()) {
         fIsHidden = taskBarAutoHide;
         if (taskBarAutoHide && fAutoHideTimer)
-            fAutoHideTimer->startTimer();
+            fAutoHideTimer->startTimer(autoHideDelay);
     }
     YWindow::handleEndPopup(popup);
 }
@@ -920,13 +930,13 @@ void TaskBar::popupWindowListMenu() {
 void TaskBar::handleDNDEnter() {
     fIsHidden = false;
     if (taskBarAutoHide && fAutoHideTimer)
-        fAutoHideTimer->startTimer();
+        fAutoHideTimer->startTimer(autoShowDelay);
 }
 
 void TaskBar::handleDNDLeave() {
     fIsHidden = taskBarAutoHide;
     if (taskBarAutoHide && fAutoHideTimer)
-        fAutoHideTimer->startTimer();
+        fAutoHideTimer->startTimer(autoHideDelay);
 }
 
 void TaskBar::popOut() {
@@ -935,7 +945,7 @@ void TaskBar::popOut() {
         updateLocation();
         fIsHidden = taskBarAutoHide;
         if (taskBarAutoHide && fAutoHideTimer)
-            fAutoHideTimer->startTimer();
+            fAutoHideTimer->startTimer(autoHideDelay);
     }
 }
 
@@ -962,6 +972,7 @@ void TaskBar::actionPerformed(YAction *action, unsigned int modifiers) {
 void TaskBar::handleCollapseButton() {
     fIsCollapsed = !fIsCollapsed;
     fCollapseButton->setText(fIsCollapsed ? "<": ">");
+    fCollapseButton->setImage(fIsCollapsed ? expandImage : collapseImage);
 
     relayout();
     showBar(true);
