@@ -305,11 +305,11 @@ bool YInputLine::handleKey(const XKeyEvent &key) {
             break;
         default:
             {
-                char c;
+                char s[16];
 
-                if (getCharFromEvent(key, &c)) {
-                    if (insertChar(c))
-                        return true;
+                if (getCharFromEvent(key, s, sizeof(s))) {
+                    replaceSelection((char *)s, strlen(s));
+                    return true;
                 }
             }
         }
@@ -563,7 +563,7 @@ void YInputLine::replaceSelection(const char *str, int len) {
         if (max < textLen)
             memcpy(newStr + min + len, fText + max, textLen - max);
         newStr[newStrLen] = 0;
-        delete fText;
+        delete[] fText;
         fText = newStr;
         curPos = markPos = min + len;
         limit();
