@@ -21,6 +21,34 @@
 /******************************************************************************/
 /******************************************************************************/
 
+struct ignore_unmap {
+    Window w;
+    ignore_unmap *next;
+
+    ignore_unmap(Window w1, ignore_unmap *n): w(w1), next(n) {}
+};
+
+static ignore_unmap *unmaps = 0;
+
+static void addIgnoreUnmap(Window w) {
+    ignore_unmap *u = new ignore_unmap(w, unmaps);
+    unmaps = u;
+}
+
+static bool ignoreUnmap(Window w) {
+    ignore_unmap **u = &unmaps;
+    while (*u) {
+        if ((*u)->w == w) {
+            ignore_unmap *u1 = *u;
+            *u = u1->next;
+            delete u1;
+            return true;
+        }
+        u = &((*u)->next);
+    }
+    return false;
+}
+
 class YAutoScroll: public YTimerListener {
 public:
     YAutoScroll();
@@ -76,8 +104,7 @@ void YAutoScroll::autoScroll(YWindow *w, bool autoScroll, const XMotionEvent *mo
     if (fAutoScrollTimer) {
         if (autoScroll) {
             if (!fAutoScrollTimer->isRunning()) {
-                fAutoScrollTimer->setInterval(autoScrollStartDelay);
-                fAutoScrollTimer->startTimer();
+                fAutoScrollTimer->startTimer(autoScrollStartDelay);
             }
         } else
             fAutoScrollTimer->stopTimer();
@@ -106,7 +133,8 @@ YWindow::YWindow(YWindow *parent, Window win):
     fFocusedWindow(0),
 
     fHandle(win), flags(0), fStyle(0), fX(0), fY(0), fWidth(1), fHeight(1),
-    fPointer(), unmapCount(0), fGraphics(0),
+    fPointer(), ///// unmapCount(0), 
+    fGraphics(0),
     fEventMask(KeyPressMask|KeyReleaseMask|FocusChangeMask|
                LeaveWindowMask|EnterWindowMask),
     fWinGravity(NorthWestGravity), fBitGravity(ForgetGravity),
@@ -346,6 +374,7 @@ void YWindow::destroy() {
     if (flags & wfCreated) {
         if (!(flags & wfDestroyed)) {
             if (!(flags & wfAdopted)) {
+                MSG(("----------------------destroy %X", fHandle));
                 XDestroyWindow(xapp->display(), fHandle);
             } else {
                 XSelectInput(xapp->display(), fHandle, NoEventMask);
@@ -392,7 +421,8 @@ void YWindow::reparent(YWindow *parent, int x, int y) {
     //flags &= ~wfVisible; // don't unmap when we get UnmapNotify
     if (flags & wfVisible) {
         flags |= wfUnmapped;
-        unmapCount++;
+        /////unmapCount++;
+        addIgnoreUnmap(handle());
     }
 
     removeWindow();
@@ -427,7 +457,8 @@ void YWindow::hide() {
         flags &= ~wfVisible;
         if (!(flags & wfNullSize)) {
             flags |= wfUnmapped;
-            unmapCount++;
+            /////unmapCount++;
+            addIgnoreUnmap(handle());
             XUnmapWindow(xapp->display(), handle());
         }
     }
@@ -589,7 +620,8 @@ void YWindow::handleEvent(const XEvent &event) {
         break;
 
     case UnmapNotify:
-        handleUnmap(event.xunmap);
+        if (!ignoreUnmap(handle()))
+            handleUnmap(event.xunmap);
         break;
 
     case ClientMessage:
@@ -640,7 +672,7 @@ void YWindow::setDoubleBuffer(bool doubleBuffer) {
     fDoubleBuffer = doubleBuffer;
 }
 
-#warning "implement expose compression"
+/// TODO #warning "implement expose compression"
 void YWindow::paintExpose(int ex, int ey, int ew, int eh) {
     Graphics &g = getGraphics();
     XRectangle r;
@@ -936,8 +968,8 @@ void YWindow::handleMap(const XMapEvent &) {
 
 void YWindow::handleUnmap(const XUnmapEvent &) {
     if (flags & wfUnmapped) {
-        unmapCount--;
-        if (unmapCount == 0)
+/////        unmapCount--;
+/////        if (unmapCount == 0)
             flags &= ~wfUnmapped;
     }
 //    else
@@ -948,8 +980,10 @@ void YWindow::handleConfigureRequest(const XConfigureRequestEvent & /*configureR
 }
 
 void YWindow::handleDestroyWindow(const XDestroyWindowEvent &destroyWindow) {
-    if (destroyWindow.window == fHandle)
+    if (destroyWindow.window == fHandle) {
         flags |= wfDestroyed;
+        while (ignoreUnmap(handle())) {}
+    }
 }
 
 void YWindow::paint(Graphics &g, const YRect &r) {
@@ -968,7 +1002,8 @@ bool YWindow::nullGeometry() {
         flags |= wfNullSize;
         if (flags & wfVisible) {
             flags |= wfUnmapped; ///!!!
-            unmapCount++;
+            /////unmapCount++;
+            addIgnoreUnmap(handle());
             XUnmapWindow(xapp->display(), handle());
         }
     } else if ((flags & wfNullSize) && !zero) {
@@ -1666,21 +1701,22 @@ unsigned int YWindow::VMod(int m) {
     return vm;
 }
 
-bool YWindow::getCharFromEvent(const XKeyEvent &key, char *c) {
-    char keyBuf[1];
+bool YWindow::getCharFromEvent(const XKeyEvent &key, char *s, int maxLen) {
+    char keyBuf[16];
     KeySym ksym;
     XKeyEvent kev = key;
 
     // FIXME:
-    int klen = XLookupString(&kev, keyBuf, 1, &ksym, NULL);
+    int klen = XLookupString(&kev, keyBuf, sizeof(keyBuf), &ksym, NULL);
 #ifndef USE_XmbLookupString
     if ((klen == 0)  && (ksym < 0x1000)) {
         klen = 1;
         keyBuf[0] = ksym & 0xFF;
     }
 #endif
-    if (klen == 1) {
-        *c = keyBuf[0];
+    if (klen >= 1 && klen < maxLen - 1) {
+        memcpy(s, keyBuf, klen);
+        s[klen] = '\0';
         return true;
     }
     return false;
